using System;
using System.IO;
using System.Text;
using System.Threading;
using System.Xml;
using XihSolutions.DotMSN;

namespace ABCommon
{
    //This class handles a conversation.  Everytime a conversation is initiated a new thread is spawned and this 
    /// class is created
    public class ConversationHandler
    {
        private Conversation convo;
        Delegate handleTransferDelegate;
        Delegate updateViewerDelegate;
        Delegate broadcastDelegate;
        Delegate statusOverrideDelegate;
        private FileStream metadata;
        private bool alive = false;
        public string remoteAccount;
        private string revisionPath;
        private Boolean fullVersion;

        public ConversationHandler(Conversation c, Delegate d, Delegate e, Delegate f, Delegate g, FileStream metadata, string revisionPath, Boolean fullVersion)
        {
            convo = c;
            convo.Switchboard.TextMessageReceived += new TextMessageReceivedEventHandler(Switchboard_TextMessageReceived);

            handleTransferDelegate = d;
            updateViewerDelegate = e;
            broadcastDelegate = f;
            statusOverrideDelegate = g;
            this.metadata = metadata;
            alive = true;
            remoteAccount = "";
            this.revisionPath = revisionPath;
            this.fullVersion = fullVersion;
        }

        void Switchboard_ContactLeft(object sender, ContactEventArgs e)
        {
            Console.WriteLine("Contact left");
        }

        void Switchboard_SessionClosed(object sender, EventArgs e)
        {
            Console.WriteLine("Session closed");
        }

        //called when a message is received
        //the message is parsed to see if it is a special command
        void Switchboard_TextMessageReceived(object sender, TextMessageEventArgs e)
        {
            Console.WriteLine(e.Sender.Name + " says:\r\n" + " " + e.Message.Text);
            remoteAccount = e.Sender.Mail;

            if (convo.SwitchboardProcessor.Connected == false)
            {
                convo.Messenger.Nameserver.RequestSwitchboard(convo.Switchboard, this);
            }

            if (e.Message.Text.ToUpper().StartsWith("GET") || e.Message.Text.ToUpper().StartsWith("REQUEST"))
            {
                string revision = "LATEST";

                if (e.Message.Text.ToUpper().TrimEnd() == "GET LATEST")
                {
                    revision = "LATEST";
                }

                else if (e.Message.Text.ToUpper().TrimEnd() == "GET TRANSCRIPT")
                {
                    revision = "TRANSCRIPT";
                }
                else if (e.Message.Text.ToUpper().TrimEnd() == "GET ORIGINAL")
                {
                    revision = "ORIGINAL";
                }

                else if (e.Message.Text.ToUpper().TrimEnd() == "GET WORKING")
                {
                    revision = "WORKING";
                }

                else if (e.Message.Text.ToUpper().TrimEnd() == "REQUEST CONTROL")
                {
                    revision = "CONTROL";
                }

                else
                {
                    try
                    {
                        String text = e.Message.Text.Trim();
                        String[] arr = text.ToUpper().Split(' ');
                        revision = arr[1];
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine(ex.Message);
                    }
                }

                try
                {
                    if (handleTransferDelegate != null)
                        handleTransferDelegate.Method.Invoke(handleTransferDelegate.Target, new object[] { e.Sender, revision });
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }

            else if (e.Message.Text.ToUpper().TrimEnd() == "ANNOUNCE EDITING")
            {
                try
                {
                    if (statusOverrideDelegate != null)
                        statusOverrideDelegate.Method.Invoke(statusOverrideDelegate.Target, new object[] { e.Sender, true });
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }

            else if (e.Message.Text.ToUpper().TrimEnd() == "ANNOUNCE EDIT DONE")
            {
                try
                {
                    if (statusOverrideDelegate != null)
                        statusOverrideDelegate.Method.Invoke(statusOverrideDelegate.Target, new object[] { e.Sender, false });
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }

            else if (e.Message.Text.ToUpper().TrimEnd() == "?" ||
                     e.Message.Text.ToUpper().TrimEnd() == "HELP")
            {
                StringBuilder message = new StringBuilder();
                message.Append("Supported Commands:\r\n");
                message.Append("GET LATEST -- The latest copy of the document will be sent to you\r\n");
                if (fullVersion)
                {
                    message.Append("GET ORIGINAL -- Gets a copy of the original document (Version #1) \r\n");
                    message.Append("GET REVISION # -- Gets a copy of the revision that was specified in the # position\r\n");
                    message.Append("GET WORKING -- Retrieves a copy of the working copy\r\n");
                    message.Append("VERSIONS -- Retrieves a list of the document versions\r\n");
                    message.Append("REQUEST CONTROL -- Sends a light version of Artifact Buddy that allows the receiver's editing activities to be displayed\r\n");
                    message.Append("ANNOUNCE EDITING -- Forces an update to the artifact's status to indicate that you're editing the document\r\n");
                    message.Append("ANNOUNCE EDIT DONE -- Update artifact's status to indicate that you're finished editing the document\r\n");
                }
                message.Append("GET TRANSCRIPT -- Gets a copy of the logs from Artifact Buddy\r\n");
                message.Append("?/HELP -- Displays this message");

                if (updateViewerDelegate != null)
                {
                    updateViewerDelegate.Method.Invoke(updateViewerDelegate.Target, new object[] { e.Sender.Name + " says:\r\n" + "   " + e.Message.Text });
                    updateViewerDelegate.Method.Invoke(updateViewerDelegate.Target, new object[] { "me" + ":" + message.ToString() });
                }
                sendMessage(message.ToString());
            }
            else if (e.Message.Text.ToUpper().StartsWith("VERSIONS"))
            {
                StringBuilder message = new StringBuilder();
                message.Append("The Versions are: \r\n");
                FileInfo[] files = new DirectoryInfo(revisionPath).GetFiles();
                for (int i = 0; i < files.Length; i++)
                {
                    String filename = files[i].Name;
                    String version = filename.Split('-')[0];
                    message.Append(version + "\r\n");
                }

                if (updateViewerDelegate != null)
                {
                    updateViewerDelegate.Method.Invoke(updateViewerDelegate.Target, new object[] { e.Sender.Name + " says:\r\n" + "   " + e.Message.Text });
                    updateViewerDelegate.Method.Invoke(updateViewerDelegate.Target, new object[] { "me" + ":" + message.ToString() });
                }

                sendMessage(message.ToString());
            }
            else
            {
                if (updateViewerDelegate != null)
                    updateViewerDelegate.Method.Invoke(updateViewerDelegate.Target, new object[] { e.Sender.Name + " says:\r\n" + "   " + e.Message.Text });

                if (broadcastDelegate != null)
                    broadcastDelegate.Method.Invoke(broadcastDelegate.Target, new object[] { e.Sender.Name + " says:\r\n" + "   " + e.Message.Text, e.Sender.Mail });
            }

            //writes the message data to the log file
            lock (metadata)
            {
                XmlTextWriter writer = new XmlTextWriter(metadata, null);
                writer.Formatting = Formatting.Indented;

                writer.WriteStartElement("message");

                writer.WriteStartElement("sender");
                writer.WriteString(e.Sender.Name);
                writer.WriteEndElement();

                writer.WriteStartElement("date");
                writer.WriteString(DateTime.Now.ToString("yyyy.MM.dd hh:mm:ss"));
                writer.WriteEndElement();

                writer.WriteStartElement("text");
                writer.WriteString(e.Message.Text);
                writer.WriteEndElement();

                writer.WriteEndElement();

                if (convo.Switchboard.Contacts.Count > 1)
                    writer.WriteEndElement();

                writer.Flush();
            }
        }

        public void run()
        {
            while (alive)
                Thread.Sleep(60000);
        }

        //used to send a message to the client on the other end of this conversation
        public void sendMessage(string message)
        {
            if (convo.SwitchboardProcessor.Connected == false)
            {
                convo.Messenger.Nameserver.RequestSwitchboard(convo.Switchboard, this);

                //sleep until the connection is established
                while (convo.SwitchboardProcessor.Connected == false)
                {
                    Thread.Sleep(20);
                }
            }

            TextMessage msg = new TextMessage(message);
            try
            {
                convo.Switchboard.SendTextMessage(msg);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
        }
    }
}
