﻿using System;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using ABCommon;
using gma.System.Windows;
using ICSharpCode.SharpZipLib.Zip;
using XihSolutions.DotMSN;
using XihSolutions.DotMSN.DataTransfer;

namespace ArtifactBuddyLite
{
    public delegate void handleTransferDelegate(Contact receiver, string revision);

    public partial class ABLiteForm : Form
    {
        CredentialStore store;
        MSNManager msnManager;
        private FileStream upLoad;
        private string name;
        private string originalFilename;
        private string latestVersion;
        private bool isOpen = false;
        private int timeCount = 0;
        private bool firstEdit = false;

        [DllImport("user32.dll")]
        static extern int GetForegroundWindow();

        [DllImport("user32.dll")]
        static extern int GetWindowText(int hWnd, StringBuilder text, int count);

        private UserActivityHook globalHook;

        public ABLiteForm()
        {
            InitializeComponent();

            Image img = pictureBox1.Image;
            pictureBox1.Image = img.GetThumbnailImage(this.pictureBox1.Size.Width, this.pictureBox1.Size.Height,
            new System.Drawing.Image.GetThumbnailImageAbort(ThumbnailCallback), IntPtr.Zero);

            try
            {
                //hook global events. This is used to detect if a user is editing the file
                globalHook = new UserActivityHook(); // crate an instance
                globalHook.KeyPress += new KeyPressEventHandler(MyKeyPress);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            store = new CredentialStore(MSNManager.CLIENT_CODE, true);

            msnManager = new MSNManager(store,
                new handleTransferDelegate(handleTransfer),
                null,
                null,
                false);
            msnManager.signedInEvent += new EventHandler(msnManager_signedInEvent);
            msnManager.connect();
            
            //get the filename
            name = store.GetFilename();

            try {
                //look to see if we can find the file
                 string currentDir=System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
                 FileInfo[] files = new DirectoryInfo(currentDir).GetFiles();
                 for (int i = 0; i < files.Length; i++)
                 {
                     if (files[i].FullName.Contains(name))
                     {
                         originalFilename = files[i].FullName;
                         this.Text = name;
                     }
                 }

                 //if not found, check one directory up
                 if (originalFilename == null)
                 {
                     char[] splitter = new char[1];
                     splitter[0] = '\\';
                     string[] temp = currentDir.Split(splitter);
                     string upOneDirectory = currentDir.Remove(currentDir.IndexOf(temp[temp.Length - 1]));
                     Console.WriteLine(temp[temp.Length - 1]);
                     files = new DirectoryInfo(upOneDirectory).GetFiles();
                     for (int i = 0; i < files.Length; i++)
                     {
                         if (files[i].FullName.Contains(name))
                         {
                             originalFilename = files[i].FullName;
                             this.Text = name;
                         }
                     }
                 }

                 //if still null the user must add it manually
                 if (originalFilename == null)
                 {
                     MessageBox.Show(name+" Not found.  Please manually select it now.", "File Error",
                         MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                     //pop up window here...
                     OpenFileDialog browseFile = new OpenFileDialog();

                     browseFile.Title = "Browse XML file";
                     if (browseFile.ShowDialog() == DialogResult.Cancel)
                         return;

                     try
                     {
                         originalFilename = browseFile.FileName;
                     }
                     catch (Exception)
                     {
                         MessageBox.Show("Error opening file", "File Error",

                         MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                     }
                 }

                 if (!Directory.Exists("versions"))
                     Directory.CreateDirectory("versions");

                 File.Copy(originalFilename, "versions//" + name, true);
                 latestVersion = "versions//" + name;

                 Console.WriteLine(originalFilename);
             }
             catch (Exception e)
             {
                 Console.WriteLine(e.Message);
             }
        }

        public bool ThumbnailCallback()
        {
            return true;
        }

        //used to monitor if the user is editing the shared file
        public void MyKeyPress(object sender, KeyPressEventArgs e)
          {
            if (originalFilename != null)
            {
                string title = GetActiveWindow();
                string file = new FileInfo(originalFilename).Name;
                file = file.Substring(0, file.IndexOf('.'));

                if (title.Contains(file))
                {
                    if (!firstEdit && msnManager.getSignedIn())
                    {
                        firstEdit = true;
                        msnManager.getMessenger().Owner.Name = Environment.UserName + " is editing " + name;
                        msnManager.getMessenger().Owner.Status = PresenceStatus.Online;
                    }
                    
                    timeCount = Environment.TickCount;
                }
            }
        }

        private string GetActiveWindow()
        {
            const int nChars = 256;
            int handle = 0;
            StringBuilder Buff = new StringBuilder(nChars);

            handle = GetForegroundWindow();

            if (GetWindowText(handle, Buff, nChars) > 0)
            {
                return (Buff.ToString());
            }
            else
                return "";
        }

        void msnManager_signedInEvent(object sender, EventArgs e)
        {
            msnManager.getMessenger().Owner.Status = PresenceStatus.Online;
            msnManager.getMessenger().Owner.Name = name;
        }

        private void handleTransfer(Contact c,string revision)
        {
            try
            {   
                string tempFilename = "";
                string suggestedFilename = "";
                Console.WriteLine(revision);
             
                if (revision.Equals("LATEST"))
                {
                    tempFilename = latestVersion;
                    suggestedFilename = Path.GetFileName(name);
                    
                }
                else if (revision.Equals("TRANSCRIPT"))
                {
                    File.Copy(msnManager.getMetaData().Name, "temp");
                    tempFilename = "temp";
                    suggestedFilename = "transcript.xml";
                    String time = DateTime.Now.ToString("ddd, hh:mmtt");
                }
                else if (revision.Equals("CONTROL"))
                {
                    store.SetFilename(name);
                    store.StoreCredentials();

                    Directory.CreateDirectory("tempZipDir");

                    File.Copy("ArtifactBuddyLite.exe","tempZipDir/ArtifactBuddyLite.exe");
                    File.Copy("credentials.xml", "tempZipDir/credentials.xml");
                    File.Copy("ABCommon.dll", "tempZipDir/ABCommon.dll");
                    File.Copy("XihSolutions.DotMSN.dll", "tempZipDir/XihSolutions.DotMSN.dll");
                    File.Copy("ICSharpCode.SharpZipLib.dll", "tempZipDir/ICSharpCode.SharpZipLib.dll");
                    FastZip zip = new FastZip();
                    zip.CreateZip("ABLite.zip", "tempZipDir", true, "");
                    Directory.Delete("tempZipDir",true);

                    tempFilename = "ABLite.zip";
                    suggestedFilename = "ABLite.zip";

                    msnManager.broadcastMessage(c.Name + " requested control.\r\n  Control is now being transferred");
                }
                
                MSNSLPHandler msnslpHandler = msnManager.getMessenger().GetMSNSLPHandler(c.Mail);
                upLoad = new FileStream(tempFilename, FileMode.Open, FileAccess.Read, FileShare.Read); //currentFilename
                P2PTransferSession session = msnslpHandler.SendInvitation(msnManager.getMessenger().Owner.Mail, c.Mail,suggestedFilename, upLoad);

                session.TransferStarted += new EventHandler(session_TransferStarted);
                session.TransferFinished += new EventHandler(session_TransferFinished);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
             
        }
        void session_TransferFinished(object sender, EventArgs e)
        {
            Console.WriteLine("Sending Xfer finished");
            upLoad.Close();
            File.Delete("temp");
            File.Delete("wc_temp");
        }

        void session_TransferStarted(object sender, EventArgs e)
        {
            Console.WriteLine("Sending Xfer started");
        }

        private void activeEditTimer_Tick(object sender, EventArgs e)
        {
            //don't bother doing anything if the file isn't open
            if (isOpen)
            {
                //check the difference between the current ticks and the last edit
                int diff = Environment.TickCount - timeCount;

                //if time count = 0, that means the file has been opened but never edited on
                if (diff > activeEditTimer.Interval || timeCount == 0)
                {
                    msnManager.getMessenger().Owner.Name = Environment.UserName + " paused editing on " + name;
                    msnManager.getMessenger().Owner.Status = PresenceStatus.Idle;
                    firstEdit = false;
                }
            }
        }

        private void fileOpenTimer_Tick(object sender, EventArgs e)
        {
            try
            {
                if (originalFilename != null)
                {
                    FileStream stream = System.IO.File.Open(originalFilename, FileMode.OpenOrCreate, FileAccess.Read, FileShare.None);
                    stream.Close();
                    isOpen = false;

                    msnManager.getMessenger().Owner.Status = PresenceStatus.Away;
                    msnManager.getMessenger().Owner.Name = name + " is closed";
                    firstEdit = false;
                }
            }
            catch
            {
                if (!isOpen)
                {
                    if (msnManager.getSignedIn())
                    {
                        msnManager.getMessenger().Owner.Status = PresenceStatus.Online;
                        msnManager.getMessenger().Owner.Name = Environment.UserName + " opened " + name;
                       isOpen = true;
                        
                    }
                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            msnManager.getMessenger().Owner.Name = name + "-" + "New Version Available";
          
            File.Copy(originalFilename, latestVersion, true);
            msnManager.broadcastMessage("New version available");
            firstEdit = false;
        }
    }
}
