using System;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Imaging;

using EasyImages;

namespace DesktopLib
{
	/// <summary>
	/// Summary description for WindowCapture.
	/// </summary>
	public class WindowCapture
	{
		private IntPtr handle;

		public WindowCapture(IntPtr windowHandle)
		{
			this.handle = windowHandle;
            
		}

		#region Internal
		#endregion

		#region Properties

		/// <summary>
		/// Returns a handle to the window curretnly being captured.
		/// </summary>
		public IntPtr Handle
		{
			get
			{
				return this.handle;
			}
		}

		#endregion

		#region Methods
		
		/// <summary>
		/// Returns an image of the window. If the window is under other windows, those top windows will be captured. If the window is minimized, the image will be gray. If the window has since been closed, the image will be red.
		/// </summary>
		public Image GrabWindow()
		{
			Image window = this.GrabWindowPhoto();
			//Image image = DesktopUtil.PhotoToBitmap(window);
			//System.Runtime.InteropServices.Marshal.ReleaseComObject(window);
			return window;

		}
        /*
		public Photo GrabWindowPhoto(float clarity, PhotoDistortStyle style)
		{
			Photo photo = this.GrabWindowPhoto();
			photo.Distort(clarity, style);
			return photo;
		}
		*/
		/// <summary>
		/// Returns a photo of the window. Use this to get a photo object so that distortion can be done.
		/// </summary>
		public Image GrabWindowPhoto()
		{
			try
			{
				Rectangle bounds = Win32.GetWindowBounds(this.handle);
				Bitmap segment = Win32.GetScreenshot(bounds.X, bounds.Y, bounds.Width, bounds.Height) as Bitmap;
				//Photo window = DesktopUtil.BitmapToPhoto(segment);

//				Photo window = new PhotoClass();
//				window.Grab(this.handle);
				return  segment;
			}
			catch (Exception e)
			{
				if (e.Message.Equals("Invalid window handle."))
				{
					throw new WindowClosedException();
				}
				else
				{
					throw new WindowMinimizedException();
				}
			}
		}
		

		#endregion
	}
}
